"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const ServiceManager_1 = require("../ServiceManager");
class AbstractFileBasedPluginManager extends ServiceManager_1.AbstractPluginManager {
    constructor(creationContext, locations, config) {
        super(creationContext, config);
        if (locations) {
            this.loadFromLocations(locations);
        }
    }
    static getPluginName(plugin) {
        if (typeof plugin === 'string') {
            return plugin;
        }
        return plugin.name;
    }
    loadFromLocations(pluginDirectories) {
        if (pluginDirectories) {
            pluginDirectories.forEach(directory => this.loadDirectory(directory));
        }
        return this;
    }
    loadDirectory(pluginDirectory) {
        const plugins = fs_1.default.readdirSync(pluginDirectory)
            .filter((fileName) => !!fileName.match(/^(?!(index)).*[^d]\.(js|ts)$/))
            .map((fileName) => fileName.replace(/\.(js|ts)$/, ''))
            .map((plugin) => {
            const Plugin = require(path_1.default.resolve(pluginDirectory, plugin));
            if (typeof Plugin === 'function') {
                return Plugin;
            }
            if (typeof Plugin.default === 'function') {
                return Plugin.default;
            }
            if (typeof Plugin[plugin] === 'function') {
                return Plugin[plugin];
            }
            throw new TypeError(`Unable to load plugin "${plugin}" due to missing constructable export.`);
        });
        this.registerPlugins(plugins);
    }
    getPlugin(plugin) {
        return this.get(plugin);
    }
    registerPlugins(plugins) {
        plugins.forEach(Plugin => this.registerPlugin(Plugin));
        return this;
    }
    registerPlugin(Plugin) {
        this.registerInvokable(Plugin, Plugin);
        this.registerAlias(Plugin.name, Plugin);
        return this;
    }
}
exports.AbstractFileBasedPluginManager = AbstractFileBasedPluginManager;
//# sourceMappingURL=AbstractFileBasedPluginManager.js.map